/**
 * @fileOverview
 * @name network.js
 * @author Wagner Müller Gegler, Roberto Oppermann Cordoni
 */

/**
 * @typedef {Object} NetworkController~InterfaceParams - Configuration parameters for a network interface.
 * @property {String} bootproto - [|none] The protocol to be used at boot time.
 * @property {String} dns1 - Name server IP address - option one.
 * @property {String} dns2 - Name server IP address - option two.
 * @property {String} gateway - Gateway device IP address.
 * @property {String} ipaddr - IP address for the interface.
 * @property {String} mode - [static|disabled|dhcp] Operation mode for the interface.
 * @property {String} name - Interface name.
 * @property {String} netmask - Network mask of this network, in dot-decimal notation.
 * @property {String} onboot - [yes|] Enable/Disable activation of interface at boot time.
 */

/**
 * @class NetworkController
 * @param {Object} $http - AngularJS service for HTTP communication.
 * @param {Object} $log - AngularJS service for logging.
 * @property {boolean[]} valid_ipaddr - Information list about the validity of the IP address field, for each network interface.
 * @property {boolean[]} valid_Netmask - Information list about the validity of the netmask address field, for each network interface.
 * @property {boolean[]} valid_gateway - Information list about the validity of the Gateway address field, for each network interface.
 * @property {boolean[]} valid_dns1 - Information list about the validity of the DNS1 address field, for each network interface.
 * @property {boolean[]} valid_dns1 - Information list about the validity of the DNS2 address field, for each network interface.
 * @property {boolean[]} loading - Information list about the validity of the DNS2 address field, for each network interface.
 * @property {NetworkController~InterfaceParams[]} interfaces - List of network parameters.
 */

angular.module('ag561app').controller('NetworkController', ['$http', '$log', '$location',
                                   function($http, $log, $location) {
  var url_network = '/api/config/network';
  var ctrl = this;
  ctrl.alert = false;
  ctrl.loading = false;
  ctrl.invalid_config = false;
  ctrl.invalid_ipaddr = false;
  ctrl.invalid_netmask = false;
  ctrl.invalid_gateway = false;
  ctrl.invalid_hostname = false;
  ctrl.invalid_dns1 = false;
  ctrl.invalid_dns2 = false;
  ctrl.invalid_http = false;
  ctrl.invalid_https = false;
  ctrl.invalid_ssh = false;
  ctrl.invalid_telnet = false;
  ctrl.invalid_rocommunity = false;
  ctrl.invalid_rwcommunity = false;
  ctrl.invalid_config_name = false;
  ctrl.invalid_config_addr = false;
  ctrl.invalid_config_netmask = false;
  ctrl.invalid_config_ip_mask = false;
  ctrl.show_snmp = false;
  ctrl.show_ip = true;
  ctrl.show_switch = false;
  ctrl.show_cpu = false;
  ctrl.show_vlans = false;
  ctrl.show_routes = false;
  ctrl.show_aliases = false;
  ctrl.vlan_indextoedit = -1;
  ctrl.alias_indextoedit = -1;
  ctrl.route_indextoedit = -1;
  ctrl.error_msg = "Invalid configuration.";
  ctrl.ip_and_mask_format = /^((^|\.)((25[0-5])|(2[0-4]\d)|(1\d\d)|([1-9]?\d))){4}(\/(?:\d|[12]\d|3[012]))?$/;
  ctrl.ipformat = new RegExp("^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$");
  ctrl.subnet_format = new RegExp("^(((255\.){3}(255|254|252|248|240|224|192|128|0+))|((255\.){2}(255|254|252|248|240|224|192|128|0+)\.0)|((255\.)(255|254|252|248|240|224|192|128|0+)(\.0+){2})|((255|254|252|248|240|224|192|128|0+)(\.0+){3}))$");

  /**
   * Clear alert message from web page, but do this only
   * if we aren't with some error. We just want to erase
   * the message of danger if all the variables are correct.
   *
   * @function clear_alert_messages
   * @memberof NetworkController
   */
  ctrl.clear_alert_messages = function() {
    if (!ctrl.invalid_config) {
      ctrl.alert_danger = false;
      ctrl.alert_success = false;
    }
    if (ctrl.alert) {
      ctrl.alert_danger = true;
      ctrl.alert = false;
    }
  };

  /**
   * Get network config and DNS parameters.
   * @function get
   * @memberof NetworkController
   */
  ctrl.get = function() {
    $http({
        method: 'GET',
        url: url_network
        }).then(function(response) {
            // First function handles success
            ctrl.ip = response.data.ip;
            ctrl.snmp = response.data.snmp;
            ctrl.switch = response.data.switch;
            ctrl.services = response.data.services;

            // Check non mandatory parameters
            if (!ctrl.switch['monitor_port'])
              ctrl.switch.monitor_port = 'none';

            // Set some variables
            ctrl.switch.port1.invalid_allowed_vids = false;
            ctrl.switch.port2.invalid_allowed_vids = false;
            updateAliasesInterfaces();
            updateVlansInterfaces();

        }, function(response) {
            // Second function handles error
            ctrl.error = response.data;
        });
    }

  /**
   * Save the new configuration.
   * @function save
   * @memberof NetworkController
   */
  ctrl.save = function() {

    // Remove variables that cannot go to API
    delete ctrl.switch.port1.invalid_allowed_vids;
    delete ctrl.switch.port2.invalid_allowed_vids;
    delete ctrl.switch.cpu.invalid_allowed_vids;

    var data = {
      'services'  : ctrl.services,
      'switch'    : ctrl.switch,
      'snmp'      : ctrl.snmp,
      'ip'        : ctrl.ip
    }

    ctrl.loading = true;
    // PUT data to API
    $http({
      method: 'PUT',
      url: url_network,
      data: data
      }).then(function(response) {
          // First function handles success
          ctrl.loading = false;
          ctrl.alert_success = true;
          ctrl.get();
      }, function(response) {
          // Second function handles error
          ctrl.error = response.data;
          ctrl.alert_danger = true;
      });
  };

/**************************************************************************************/

  /**
   * Add new vlan to object 'ctrl.ip.vlans'
   *
   * @function vlanAdd
   * @memberof NetworkController
   */
   ctrl.vlanAdd = function() {

    if (vlanLimitWasReached()) {
      ctrl.error_msg = "Vlans limit was reached (16)";
      ctrl.alert = true;
      return;
    }

    var vlan = {'name': '',
                'mode': 'static',
                'addr': '',
                'mask': ''
    };

    ctrl.invalid_config_name = true;
    ctrl.invalid_config_addr = true;
    ctrl.invalid_config_netmask = true;

    ctrl.ip.vlans.push(vlan);
    ctrl.vlan_indextoedit = ctrl.ip.vlans.indexOf(vlan);
    updateVlansInterfaces();
    __checkInvalidConfig();
  };

  /**
   * Remove specific vlan from main object 'ctrl.ip.vlans'
   *
   * @function vlanDelete
   * @memberof NetworkController
   */
   ctrl.vlanDelete = function(index) {
    ctrl.ip.vlans.splice(index, 1);
    ctrl.invalid_config_name = false;
    ctrl.invalid_config_addr = false;
    ctrl.invalid_config_netmask = false;
    updateVlansInterfaces();
    __checkInvalidConfig();
  }

  /**
   * Edit specific vlan from main object 'ctrl.ip.vlans'
   *
   * @function vlanEdit
   * @memberof NetworkController
   */
  ctrl.vlanEdit = function(index) {
    ctrl.vlan_indextoedit = index;
  }

  /**
   * Confirm vlan edition
   *
   * @function vlanConfirm
   * @memberof NetworkController
   */
  ctrl.vlanConfirm = function() {
    ctrl.vlan_indextoedit = -1;
    updateVlansInterfaces();
  }

  /**
   * Update number of vlans
   *
   * @function updateVlansInterfaces
   * @memberof NetworkController
   */
  function updateVlansInterfaces() {
    ctrl.number_of_vlans = 0;
    for (var a = 0; a < ctrl.ip.vlans.length; a++) {
      ctrl.number_of_vlans++;
    }
  }

  /**
   * Limit the number of vlans
   *
   * @function vlanLimitWasReached
   * @memberof NetworkController
   */
  function vlanLimitWasReached() {
    console.log(ctrl.number_of_vlans)
    if (ctrl.number_of_vlans >= 16)
      return true;

    return false;
  }

    /**
   * Check if vlan ID is valid
   *
   * @function validateVlanId
   * @memberof NetworkController
   */
     ctrl.validateVlanId = function(vlan, index) {
      ctrl.invalid_config_name = true;

      if (_alreadyExists(ctrl.ip.vlans, vlan.name, index) || !vlan.name) {
        __checkInvalidConfig();
        return;
      }

      if (vlan.name.length == 0 || typeof(parseInt(vlan.name)) != 'number') {
         __checkInvalidConfig();
         return;
      }

      ctrl.invalid_config_name = false;
      __checkInvalidConfig();
    }

    /**
   * Check if vlan is DHCP
   *
   * @function validateVlanMode
   * @memberof NetworkController
   */
     ctrl.validateVlanMode = function(vlan) {

      if (vlan.mode == 'dhcp') {
        ctrl.invalid_config_addr = false;
        ctrl.invalid_config_netmask = false;
      }

      __checkInvalidConfig();
    }

/**************************************************************************************/

  /**
   * Add new alias to object 'ctrl.ip.aliases'
   *
   * @function aliasAdd
   * @memberof NetworkController
   */
   ctrl.aliasAdd = function() {

    if (aliasLimitWasReached()) {
      ctrl.error_msg = "Aliases limit was reached (16)";
      ctrl.alert = true;
      return;
    }

    var alias = {'name': '',
                'addr': '',
                'mask': ''
    };

    ctrl.invalid_config_name = true;
    ctrl.invalid_config_addr = true;
    ctrl.invalid_config_netmask = true;

    ctrl.ip.aliases.push(alias);
    ctrl.alias_indextoedit = ctrl.ip.aliases.indexOf(alias);
    updateAliasesInterfaces();
    __checkInvalidConfig();
  };

  /**
   * Remove specific alias from main object 'ctrl.ip.aliases'
   *
   * @function aliasDelete
   * @memberof NetworkController
   */
   ctrl.aliasDelete = function(index) {
    ctrl.ip.aliases.splice(index, 1);
    ctrl.invalid_config_name = false;
    ctrl.invalid_config_addr = false;
    ctrl.invalid_config_netmask = false;
    updateAliasesInterfaces();
    __checkInvalidConfig();
  }

  /**
   * Edit specific alias from main object 'ctrl.ip.aliases'
   *
   * @function aliasEdit
   * @memberof NetworkController
   */
  ctrl.aliasEdit = function(index) {
    ctrl.alias_indextoedit = index;
  }

  /**
   * Confirm alias edition
   *
   * @function aliasConfirm
   * @memberof NetworkController
   */
  ctrl.aliasConfirm = function() {
    ctrl.alias_indextoedit = -1;
    updateAliasesInterfaces();
  }

  /**
   * Update number of aliases
   *
   * @function updateAliasesInterfaces
   * @memberof NetworkController
   */
  function updateAliasesInterfaces() {
    ctrl.number_of_aliases = 0;
    for (var a = 0; a < ctrl.ip.aliases.length; a++) {
      ctrl.number_of_aliases++;
    }
  }

  /**
   * Limit the number of aliases
   *
   * @function aliasLimitWasReached
   * @memberof NetworkController
   */
  function aliasLimitWasReached() {

    if (ctrl.number_of_aliases >= 16)
      return true;

    return false;
  }

    /**
   * Check if alias ID is valid
   *
   * @function validateAliasId
   * @memberof NetworkController
   */
     ctrl.validateAliasId = function(alias, index) {
      ctrl.invalid_config_name = true;

      if (_alreadyExists(ctrl.ip.aliases, alias.name, index)) {
        __checkInvalidConfig();
        return;
      }

      if (alias.name.length == 0) {
         __checkInvalidConfig();
         return;
      }

      ctrl.invalid_config_name = false;
      __checkInvalidConfig();
    }

/**************************************************************************************/

  /**
   * Add new route to object 'ctrl.ip.routes'
   *
   * @function routeAdd
   * @memberof NetworkController
   */
   ctrl.routeAdd = function() {
    var route = { 'name': '',
                  'dest': '',
                  'gw': ''
    };

    ctrl.invalid_config_name = true;
    ctrl.invalid_config_addr = true;
    ctrl.invalid_config_ip_mask = true;

    ctrl.ip.routes.push(route);
    ctrl.route_indextoedit = ctrl.ip.routes.indexOf(route);
    __checkInvalidConfig();
  };

  /**
   * Remove specific route from main object 'ctrl.ip.routes'
   *
   * @function routeDelete
   * @memberof NetworkController
   */
   ctrl.routeDelete = function(index) {
    ctrl.ip.routes.splice(index, 1);
    ctrl.invalid_config_name = false;
    ctrl.invalid_config_addr = false;
    ctrl.invalid_config_ip_mask = false;
    __checkInvalidConfig();
  }

  /**
   * Edit specific route from main object 'ctrl.ip.routes'
   *
   * @function routeEdit
   * @memberof NetworkController
   */
  ctrl.routeEdit = function(index) {
    ctrl.route_indextoedit = index;
  }

  /**
   * Confirm route edition
   *
   * @function routeConfirm
   * @memberof NetworkController
   */
  ctrl.routeConfirm = function() {
    ctrl.route_indextoedit = -1;
  }

  /**
   * Check if route ID is valid
   *
   * @function validateRouteId
   * @memberof NetworkController
   */
    ctrl.validateRouteId = function(route, index) {
      ctrl.invalid_config_name = true;

      if (_alreadyExists(ctrl.ip.routes, route.name, index)) {
        __checkInvalidConfig();
        return;
      }

      if (route.name.length == 0) {
         __checkInvalidConfig();
         return;
      }

      ctrl.invalid_config_name = false;
      __checkInvalidConfig();
    }

/**************************************************************************************/

  /**
   * Check if some configuration is invalid. If true, then disable the
   * button to 'save' config and show the 'danger message'.
   *
   * @function __checkInvalidConfig
   * @memberof NetworkController
   */
  __checkInvalidConfig = function() {
    if (ctrl.invalid_ipaddr || ctrl.invalid_netmask || ctrl.invalid_gateway ||
        ctrl.invalid_dns1 || ctrl.invalid_dns2 ||
        ctrl.invalid_pppoe_username || ctrl.invalid_pppoe_password ||
        ctrl.invalid_ntp || ctrl.invalid_syslog || ctrl.invalid_allowed_vids ||
        ctrl.invalid_rocommunity || ctrl.invalid_rwcommunity ||
        ctrl.invalid_config_name || ctrl.invalid_config_addr ||
        ctrl.invalid_config_netmask || ctrl.invalid_config_ip_mask ||
        ctrl.invalid_http || ctrl.invalid_https ||
        ctrl.invalid_ssh || ctrl.invalid_telnet ) {
      ctrl.invalid_config = true;
      ctrl.alert_danger = true;
      ctrl.error_msg = "Invalid configuration.";
    } else {
      ctrl.invalid_config = false;
      ctrl.alert_danger = false;
    }
  };

  ctrl.validateIpConfig = function() {
      /* Check static IP address and netmask */
      ctrl.invalid_ipaddr = ctrl.ip.type == 'static' && ctrl.ip.addr && !ctrl.ip.addr.match(ctrl.ipformat);
      ctrl.invalid_netmask = ctrl.ip.type == 'static' && ctrl.ip.mask && !ctrl.ip.mask.match(ctrl.ipformat);

      /* Default Gateway is optional. Only check if it is not empty */
      ctrl.invalid_gateway = ctrl.ip.gateway && !ctrl.ip.gateway.match(ipformat);

      /* DNS Servers are optional. Only check if it is not empty */
      ctrl.invalid_dns1 = ctrl.ip.type != 'pppoe' && ctrl.ip.dns[0] && !ctrl.ip.dns[0].match(ctrl.ipformat);
      ctrl.invalid_dns2 = ctrl.ip.type != 'pppoe' && ctrl.ip.dns[1] && !ctrl.ip.dns[1].match(ctrl.ipformat);

      /* Validate PPPoE username and password */
      ctrl.invalid_pppoe_username = ctrl.ip.type == 'pppoe' && !ctrl.ip.pppoe_username.match(/^([\S]+)$/);
      ctrl.invalid_pppoe_password = ctrl.ip.type == 'pppoe' && !ctrl.ip.pppoe_password.match(/^([\S]+)$/);

      __checkInvalidConfig();
  };

  /* Server eg.: aligera:1234 or 192.168.2.1:1234 or 10.10.10.1 */
  var server_port_regex = /^([\w.-]+)(:[\d]+)?$/;
  var community_regex = /^[\w\-.]+$/;

  ctrl.validateNTP = function() {
    ctrl.invalid_ntp = ctrl.ip.ntp && !ctrl.ip.ntp.match(server_port_regex);
     __checkInvalidConfig();
  };

  ctrl.validateSyslog = function() {
    ctrl.invalid_syslog = ctrl.ip.syslog && !ctrl.ip.syslog.match(server_port_regex);
    __checkInvalidConfig();
  };

  ctrl.validateRoCommunity = function() {
    ctrl.invalid_rocommunity = ctrl.snmp.rocommunity && !ctrl.snmp.rocommunity.match(community_regex);
     __checkInvalidConfig();
  };


  ctrl.validateRwCommunity = function() {
    ctrl.invalid_rwcommunity = ctrl.snmp.rwcommunity && !ctrl.snmp.rwcommunity.match(community_regex);
     __checkInvalidConfig();
  };

  ctrl.validateHostname = function() {
    var regex = /^[a-zA-Z0-9][a-zA-Z0-9\.\-]*$/;

    ctrl.invalid_hostname = ctrl.ip.hostname.match(regex) ? false : true;

    __checkInvalidConfig();
  };

  _alreadyExists = (config, name, index) => {

    for (let v = 0; v < config.length; v++) {
      if ((config[v].name == name) && index != v)
        return true;
    }

    return false;
  }

  /**
   * Check if IP is valid
   *
   * @function validateIp
   * @memberof NetworkController
   */
  ctrl.validateIp = function(config) {
    ctrl.invalid_config_addr = ((!config.addr || config.addr == "") ||
      (config.addr && config.addr.match(ctrl.ipformat))) ? false : true;
      __checkInvalidConfig();
  };


  /**
   * Check if mask is valid
   *
   * @function validateMask
   * @memberof NetworkController
   */
  ctrl.validateMask = function(config) {
    ctrl.invalid_config_netmask = ((!config.mask || config.mask == "") ||
      (config.mask && config.mask.match(ctrl.subnet_format))) ? false : true;

    __checkInvalidConfig();
  };

  /**
   * Check if Ip/mask (10.0.0.1/24) is valid
   *
   * @function validateIpMask
   * @memberof NetworkController
   */
  ctrl.validateIpMask = function(config){
    ctrl.invalid_config_ip_mask = ((!config.dest || config.dest == "") ||
    (config.dest && ctrl.ip_and_mask_format.test(config.dest))) ? false : true;

    __checkInvalidConfig();
  };

  /**
   * Check if port allowed VLAN IDs are valid. If not valid, sets a control variable called
   * invalid_allowed_vids, for the port passed as parameter. Also, calls __checkInvalidConfig()
   * function to disable the save button.
   *
   * @function validateAllowedVids
   * @memberof NetworkController
   */
  ctrl.validateAllowedVids = function(port) {
    port.invalid_allowed_vids = false;

    // Split array by spaces or commas
    var vids = port.allowed_vids.split(/[ ,]+/);
    for (var i = 0; i < vids.length; i++) {
      vid = parseInt(vids[i], 10);
      // Check VID range and if it doesnt have letter in it
      if (vid < 2 || vid > 4094 || vids[i].search(/[a-zA-Z]+/) >= 0)
        port.invalid_allowed_vids = true;
    }

    if (port.invalid_allowed_vids == true)
      ctrl.invalid_allowed_vids = true;
    else
      ctrl.invalid_allowed_vids = false;

    __checkInvalidConfig();
  };

  ctrl.validatePort = function(service, port) {
    var invalid_port = true;

    if (checkPort(service, port)) {
      invalid_port = false;
    }

    if (service === 'http')
      ctrl.invalid_http = invalid_port;
    if (service === 'https')
      ctrl.invalid_https = invalid_port;
    if (service === 'ssh')
      ctrl.invalid_ssh = invalid_port;
    if (service === 'telnet')
      ctrl.invalid_telnet = invalid_port;

    __checkInvalidConfig();
  };

  function checkPort(service, port) {

    for (const [ key, value ] of Object.entries(ctrl.services)) {
      if (port === value && service != key) {
        return false;
      }
    }

    if (port > 0 && port < 65536)
      return true;

    return false;
  };

  ctrl.get();

}]);
